@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2012 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,16 +24,68 @@ Methods:
   parse_connection()         Parse connection parameters
 """
 
+import copy
 import optparse
 import re
 
 from .. import VERSION_FRM
 from mysql.utilities.exception import UtilError
+from optparse import Option as CustomOption, OptionValueError as ValueError
 
-_PERMITTED_FORMATS = ["GRID", "TAB", "CSV", "VERTICAL"]
+_PERMITTED_FORMATS = ["grid", "tab", "csv", "vertical"]
 _PERMITTED_DIFFS = ["unified", "context", "differ"]
+_PERMITTED_RPL_DUMP = ["master", "slave"]
 
-def setup_common_options(program_name, desc_str, usage_str, append=False):
+def prefix_check_choice(option, opt, value):
+    """Check option values using case insensitive prefix compare
+    
+    This method checks to see if the value specified is a prefix of one of the
+    choices. It converts the string provided by the user (value) to lower case
+    to permit case insensitive comparison of the user input. If multiple
+    choices are found for a prefix, an error is thrown. If the value being
+    compared does not match the list of choices, an error is thrown.
+    
+    option[in]             Option class instance
+    opt[in]                option name
+    value[in]              the value provided by the user
+    
+    Returns string - valid option chosen
+    """
+    choices = ", ".join(map(repr, option.choices)) # String of choices
+
+    # Get matches for prefix given
+    alts = [alt for alt in option.choices if alt.startswith(value.lower())]
+    if len(alts) == 1:   # only 1 match 
+       return alts[0]
+    elif len(alts) > 1:  # multiple matches
+        raise ValueError(("option %s: there are multiple prefixes matching: "
+                          "%r (choose from %s)") % (opt, value, choices))
+        
+    # Doesn't match. Show user possible choices.
+    raise ValueError(("option %s: invalid choice: %r (choose from %s)")
+                     % (opt, value, choices))
+
+
+class CaseInsensitiveChoicesOption(CustomOption):
+    """Case insensitive choices option class
+    
+    This is an extension of the Option class. It replaces the check_choice
+    method with the prefix_check_choice() method above to provide
+    shortcut aware choice selection. It also ensures the choice compare is
+    done with a case insensitve test.
+    """
+    TYPE_CHECKER = copy.copy(CustomOption.TYPE_CHECKER)
+    TYPE_CHECKER["choice"] = prefix_check_choice
+
+    def __init__(self, *opts, **attrs):
+        if 'choices' in attrs:
+            attrs['choices'] = [ attr.lower() for attr in attrs['choices'] ]
+        CustomOption.__init__(self, *opts, **attrs)
+
+
+def setup_common_options(program_name, desc_str, usage_str,
+                         append=False, server=True,
+                         server_default="root@localhost:3306"):
     """Setup option parser and options common to all MySQL Utilities.
 
     This method creates an option parser and adds options for user
@@ -45,6 +97,10 @@ def setup_common_options(program_name, desc_str, usage_str, append=False):
     usage_str[in]      A brief usage example
     append[in]         If True, allow --server to be specified multiple times
                        (default = False)
+    server[in]         If True, add the --server option
+                       (default = True)
+    server_default[in] Default value for option
+                       (default = "root@localhost:3306")
 
     Returns parser object
     """
@@ -53,27 +109,32 @@ def setup_common_options(program_name, desc_str, usage_str, append=False):
         version=VERSION_FRM.format(program=program_name),
         description=desc_str,
         usage=usage_str,
-        add_help_option=False)
-    parser.add_option("--help", action="help")
+        add_help_option=False,
+        option_class=CaseInsensitiveChoicesOption)
+    parser.add_option("--help", action="help", help="display a help message "
+                      "and exit")
 
-    # Connection information for the first server
-    if append:
-        parser.add_option("--server", action="append", dest="server",
-                          help="connection information for the server in " + \
-                          "the form: <user>:<password>@<host>:<port>:<socket>")
-    else:
-        parser.add_option("--server", action="store", dest="server",
-                          type = "string", default="root@localhost:3306",
-                          help="connection information for the server in " + \
-                          "the form: <user>:<password>@<host>:<port>:<socket>")
+    if server:
+        # Connection information for the first server
+        if append:
+            parser.add_option("--server", action="append", dest="server",
+                              help="connection information for the server in "
+                              "the form: <user>:<password>@<host>:<port>:"
+                              "<socket>")
+        else:
+            parser.add_option("--server", action="store", dest="server",
+                              type = "string", default=server_default,
+                              help="connection information for the server in "
+                              "the form: <user>:<password>@<host>:<port>:"
+                              "<socket>")
 
     return parser
 
 
 _SKIP_VALUES = (
-    "TABLES","VIEWS","TRIGGERS","PROCEDURES",
-    "FUNCTIONS","EVENTS","GRANTS","DATA",
-    "CREATE_DB"
+    "tables","views","triggers","procedures",
+    "functions","events","grants","data",
+    "create_db"
 )
 
 def add_skip_options(parser):
@@ -84,8 +145,8 @@ def add_skip_options(parser):
     parser.add_option("--skip", action="store", dest="skip_objects",
                       default=None, help="specify objects to skip in the "
                       "operation in the form of a comma-separated list (no "
-                      "spaces). Valid values = TABLES, VIEWS, TRIGGERS, PROC"
-                      "EDURES, FUNCTIONS, EVENTS, GRANTS, DATA, CREATE_DB")
+                      "spaces). Valid values = tables, views, triggers, proc"
+                      "edures, functions, events, grants, data, create_db")
 
 
 def check_skip_options(skip_list):
@@ -102,36 +163,32 @@ def check_skip_options(skip_list):
     if skip_list is not None:
         items = skip_list.split(",")
         for object in items:
-            if object.upper() in _SKIP_VALUES:
-                new_skip_list.append(object.upper())
+            obj = object.lower()
+            if obj in _SKIP_VALUES:
+                new_skip_list.append(obj)
             else:
                 raise UtilError("The value %s is not a valid value for "
-                                     "--skip." % object)
+                                "--skip." % object)
     return new_skip_list
 
 
-def check_format_option(option, sql=False, initials=False):
-    """Check format option for validity.
+def add_format_option(parser, help_text, default_val, sql=False):
+    """Add the format option.
     
-    option[in]        value specified
-    sql[in]           if True, add 'SQL' format
-                      default=False
-    initials[in]      if True, add initial caps for compares
+    parser[in]        the parser instance
+    help_text[in]     help text
+    default_val[in]   default value
+    sql[in]           if True, add 'sql' format
                       default=False
     
     Returns corrected format value
     """
     formats = _PERMITTED_FORMATS
     if sql:
-        formats.append('SQL')
-    candidates = [ f for f in formats if f.startswith(option.upper()) ]
-    if len(candidates) > 1:
-        message = ''.join([value, "is ambigous. Alternatives:"] + candidates)
-        raise UtilError(message)
-    if len(candidates) == 0:
-        raise UtilError(option + " is not a valid format option.")
-
-    return candidates[0]
+        formats.append('sql')
+    parser.add_option("-f", "--format", action="store", dest="format",
+                      default=default_val, help=help_text, type="choice",
+                      choices=formats)
 
 
 def add_verbosity(parser, quiet=True):
@@ -143,11 +200,12 @@ def add_verbosity(parser, quiet=True):
 
     """
     parser.add_option("-v", "--verbose", action="count", dest="verbosity",
-                      help="Control how much information is displayed. "
+                      help="control how much information is displayed. "
                       "e.g., -v = verbose, -vv = more verbose, -vvv = debug")
     if quiet:
         parser.add_option("-q", "--quiet", action="store_true", dest="quiet",
-                          help="Turn off all messages for quiet execution.")
+                          help="turn off all messages for quiet execution.",
+                          default=False)
 
 
 def check_verbosity(options):
@@ -166,7 +224,7 @@ def add_changes_for(parser):
     parser[in]        the parser instance
     """
     parser.add_option("--changes-for", action="store", dest="changes_for",
-                      type="choice", default="server1", help="Specify the "
+                      type="choice", default="server1", help="specify the "
                       "server to show transformations to match the other "
                       "server. For example, to see the transformation for "
                       "transforming server1 to match server2, use "
@@ -215,11 +273,11 @@ def add_engines(parser):
     """
     # Add engine
     parser.add_option("--new-storage-engine", action="store", dest="new_engine",
-                      default=None, help="Change all tables to use this "\
+                      default=None, help="change all tables to use this "\
                       "storage engine if storage engine exists on the destination.")
     # Add default storage engine
     parser.add_option("--default-storage-engine", action="store",
-                      dest="def_engine", default=None, help="Change all "
+                      dest="def_engine", default=None, help="change all "
                       "tables to use this storage engine if the original "
                       "storage engine does not exist on the destination.")
 
@@ -297,7 +355,7 @@ def add_locking(parser):
     parser.add_option("--locking", action="store", dest="locking",
                       type="choice", default="snapshot", 
                       choices=['no-locks', 'lock-all', 'snapshot'],
-                      help="Choose the lock type for the operation: no-locks "
+                      help="choose the lock type for the operation: no-locks "
                       "= do not use any table locks, lock-all = use table "
                       "locks but no transaction and no consistent read, "
                       "snaphot (default): consistent read using a single "
@@ -310,10 +368,171 @@ def add_regexp(parser):
     parser[in]        the parser instance
     """
     parser.add_option("-G", "--basic-regexp", "--regexp", dest="use_regexp",
-                      action="store_true", default=False, help="Use 'REGEXP' "
+                      action="store_true", default=False, help="use 'REGEXP' "
                       "operator to match pattern. Default is to use 'LIKE'.")
 
 
+def add_rpl_user(parser, default_val="rpl:rpl"):
+    """Add the --rpl-user option.
+
+    parser[in]        the parser instance
+    default_val[in]   default value for user, password
+                      Default = rpl, rpl
+    """
+    parser.add_option("--rpl-user", action="store", dest="rpl_user",
+                      type = "string", default=default_val,
+                      help="the user and password for the replication " 
+                           "user requirement - e.g. rpl:passwd " 
+                           "- default = %default")
+
+
+def add_rpl_mode(parser, do_both=True, add_file=True):
+    """Add the --rpl and --rpl-file options.
+
+    parser[in]        the parser instance
+    do_both[in]       if True, include the "both" value for the --rpl option
+                      Default = True
+    add_file[in]      if True, add the --rpl-file option
+                      Default = True
+    """
+    rpl_mode_both = ""
+    rpl_mode_options = _PERMITTED_RPL_DUMP
+    if do_both:
+        rpl_mode_options.append("both")
+        rpl_mode_both = ", and 'both' = include 'master' and 'slave' " + \
+                        "options where applicable"
+    parser.add_option("--rpl", "--replication", dest="rpl_mode", action="store",
+                      help="include replication information. Choices = 'master'"
+                      " = include the CHANGE MASTER command using source "
+                      "server as the mastert, 'slave' = include the CHANGE "
+                      "MASTER command using the destination server's master "
+                      "information%s." % rpl_mode_both,
+                      choices=rpl_mode_options)
+    if add_file:
+        parser.add_option("--rpl-file", "--replication-file", dest="rpl_file",
+                          action="store", help="path and file name to place the "
+                          "replication information generated. Valid on if the "
+                          "--rpl option is specified.")
+    
+    
+def check_rpl_options(parser, options):
+    """Check replication dump options for validity
+    
+    This method ensures the optional --rpl-* options are valid only when
+    --rpl is specified.
+    
+    parser[in]        the parser instance
+    options[in]       command options
+    """
+    if options.rpl_mode is None:
+        errors = []
+        if parser.has_option("--comment-rpl") and options.rpl_file is not None:
+            errors.append("--rpl-file")
+
+        if options.rpl_user is not None:
+            errors.append("--rpl-user")
+                    
+        # It's Ok if the options do not include --comment-rpl
+        if parser.has_option("--comment-rpl") and options.comment_rpl:
+            errors.append("--comment-rpl")
+        
+        if len(errors) > 1:
+            num_opt_str = "s"
+        else:
+            num_opt_str = ""
+
+        if len(errors) > 0:
+            parser.error("The %s option%s must be used with the --rpl "
+                         "option." % (", ".join(errors), num_opt_str))
+            
+
+    
+def add_failover_options(parser):
+    """Add the common failover options.
+    
+    This adds the following options:
+    
+      --candidates
+      --discover-slaves-login
+      --exec-after
+      --exec-before
+      --log
+      --log-age
+      --master
+      --max-position
+      --ping
+      --seconds-behind
+      --slaves
+      --timeout
+      
+    parser[in]        the parser instance
+    """
+    parser.add_option("--candidates", action="store", dest="candidates",
+                      type="string", default=None,
+                      help="connection information for candidate slave servers "
+                      "for failover in the form: <user>:<password>@<host>:"
+                      "<port>:<socket>. Valid only with failover command. "
+                      "List multiple slaves in comma-separated list.")
+
+    parser.add_option("--discover-slaves-login", action="store", dest="discover",
+                      default=None, type="string", help="at startup, query "
+                      "master for all registered slaves and use the user name "
+                      "and password specified to connect. Supply the user and "
+                      "password in the form user:password. For example, "
+                      "--discover=joe:secret will use 'joe' as the user and "
+                      "'secret' as the password for each discovered slave.")
+
+    parser.add_option("--exec-after", action="store", dest="exec_after",
+                      default=None, type="string", help="name of script to "
+                      "execute after failover or switchover")
+    
+    parser.add_option("--exec-before", action="store", dest="exec_before",
+                      default=None, type="string", help="name of script to "
+                      "execute before failover or switchover")
+
+    parser.add_option("--log", action="store", dest="log_file", default=None,
+                      type="string", help="specify a log file to use for "
+                      "logging messages")
+    
+    parser.add_option("--log-age", action="store", dest="log_age", default=7,
+                      type="int", help="specify maximum age of log entries in "
+                      "days. Entries older than this will be purged on startup. "
+                      "Default = 7 days.")
+
+    parser.add_option("--master", action="store", dest="master", default=None,
+                      type="string", help="connection information for master "
+                      "server in the form: <user>:<password>@<host>:<port>:"
+                      "<socket>")
+    
+    parser.add_option("--max-position", action="store", dest="max_position",
+                      default=0, type="int", help="Used to detect slave "
+                      "delay. The maximum difference between the master's "
+                      "log position and the slave's reported read position of "
+                      "the master. A value greater than this means the slave "
+                      "is too far behind the master. Default is 0.")
+    
+    parser.add_option("--ping", action="store", dest="ping", default=None,
+                      help="Number of ping attempts for detecting downed "
+                      "server.")
+    
+    parser.add_option("--seconds-behind", action="store", dest="max_delay",
+                      default=0, type="int", help="Used to detect slave "
+                      "delay. The maximum number of seconds behind the master "
+                      "permitted before slave is considered behind the master. "
+                      "Default is 0.")
+
+    parser.add_option("--slaves", action="store", dest="slaves",
+                      type="string", default=None,
+                      help="connection information for slave servers in " 
+                      "the form: <user>:<password>@<host>:<port>:<socket>. "
+                      "List multiple slaves in comma-separated list.")
+    
+    parser.add_option("--timeout", action="store", dest="timeout", default=3,
+                      help="Maximum timeout in seconds to wait for each "
+                      "replication command to complete. For example, timeout "
+                      "for slave waiting to catch up to master. Default = 3.")
+
+
 def obj2sql(obj):
     """Convert a Python object to an SQL object.
 
