@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2012 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1007,7 +1007,7 @@ class Table(object):
         This shows the best (first n) or worst (last n) performing queries
         for a given table.
 
-        format[in]         format out output = SQL, TABLE, TAB, CSV
+        format[in]         format out output = sql, table, tab, csv
         limit[in]          number to limit the display
         best[in]           (optional) if True, print best performing indexes
                                       if False, print worst performing indexes
@@ -1067,20 +1067,20 @@ class Table(object):
                   (type, self.table)
             cols = ("database", "table", "name", "column", "sequence",
                     "num columns", "cardinality", "est. rows", "percent")
-            print_list(sys.stdout, format, cols, rows, False)
+            print_list(sys.stdout, format, cols, rows)
 
 
     def __print_index_list(self, indexes, format, no_header=False):
         """Print the list of indexes
 
         indexes[in]        list of indexes to print
-        format[in]         format out output = SQL, TABLE, TAB, CSV
+        format[in]         format out output = sql, table, tab, csv
         no_header[in]      (optional) if True, do not print the header
         """
 
         from mysql.utilities.common.format import print_list
 
-        if format == "SQL":
+        if format == "sql":
             for index in indexes:
                 index.print_index_sql()
         else:
@@ -1094,11 +1094,11 @@ class Table(object):
     def print_indexes(self, format):
         """Print all indexes for this table
 
-        format[in]         format out output = SQL, TABLE, TAB, CSV
+        format[in]         format out output = sql, table, tab, csv
         """
 
         print "# Showing indexes from %s:\n#" % (self.table)
-        if format == "SQL":
+        if format == "sql":
             self.__print_index_list(self.btree_indexes, format)
             self.__print_index_list(self.hash_indexes, format, False)
             self.__print_index_list(self.rtree_indexes, format, False)
